import { SerializableValue } from '@previewjs/serializable-values';
import { TypeAnalyzer, ValueType, CollectedTypes } from '@previewjs/type-analyzer';
import { Reader } from '@previewjs/vfs';
import { Logger } from 'pino';

type ComponentAnalyzer = {
    typeAnalyzer: Omit<TypeAnalyzer, "dispose">;
    detectComponents: (filePaths: string[]) => Promise<Component[]>;
    dispose: () => void;
};
type ComponentAnalyzerFactory = (options: {
    rootDir: string;
    reader?: Reader;
    logger?: Logger;
}) => ComponentAnalyzer;
interface BaseComponent {
    componentId: string;
    offsets: [start: number, end: number];
}
type Component = FrameworkComponent | StoryComponent;
interface FrameworkComponent extends BaseComponent {
    kind: "component";
    exported: boolean;
    extractProps: () => Promise<ComponentProps>;
}
interface ComponentProps {
    props: ValueType;
    types: CollectedTypes;
}
interface StoryComponent extends BaseComponent {
    kind: "story";
    args: {
        start: number;
        end: number;
        value: SerializableValue;
    } | null;
    associatedComponent: BasicFrameworkComponent | null;
}
type BasicFrameworkComponent = Pick<FrameworkComponent, "componentId" | "extractProps">;

declare function generateComponentId(options: {
    filePath: string;
    name: string;
}): string;
declare function decodeComponentId(componentId: string): {
    filePath: string;
    name: string;
};

type ComponentAnalyzerFactoryOptions = Parameters<ComponentAnalyzerFactory>[0];
declare function factoryWithDefaultOptions(factory: (options: Required<ComponentAnalyzerFactoryOptions>) => ComponentAnalyzer): ComponentAnalyzerFactory;

export { BaseComponent, BasicFrameworkComponent, Component, ComponentAnalyzer, ComponentAnalyzerFactory, ComponentProps, FrameworkComponent, StoryComponent, decodeComponentId, factoryWithDefaultOptions, generateComponentId };
