import { createFileSystemReader } from '@previewjs/vfs';
import createLogger from 'pino';
import prettyLogger from 'pino-pretty';

function generateComponentId(options) {
  return `${options.filePath.replace(/\\/g, "/")}:${options.name}`;
}
function decodeComponentId(componentId) {
  const colonPosition = componentId.lastIndexOf(":");
  if (colonPosition === -1) {
    throw new Error(`Invalid component ID: "${componentId}"`);
  }
  const [filePath, componentName] = componentId.split(":");
  return {
    filePath,
    name: componentName
  };
}

function factoryWithDefaultOptions(factory) {
  return (options) => factory({
    ...options,
    reader: options.reader || createFileSystemReader(),
    logger: options.logger || createLogger({ level: "debug" }, prettyLogger({ colorize: true }))
  });
}

export { decodeComponentId, factoryWithDefaultOptions, generateComponentId };
